{%- extends "basic/layout.html" %}

{%- block extrahead %}
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Neuton&amp;subset=latin" type="text/css" media="screen" charset="utf-8" />
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Nobile:regular,italic,bold,bolditalic&amp;subset=latin" type="text/css" media="screen" charset="utf-8" />
{% endblock %}

{%- block sidebarlogo %}
{%- if logo %}
  <div class="logo">
    <a href="{{ pathto(master_doc) }}">
      <img class="logo" src="{{ pathto('_static/' + logo, 1) }}" alt="Logo"/>
    </a>
  </div>
{%- endif %}
{%- endblock %}

